package applications.CAcollages;

import java.util.*;
import java.awt.geom.*;

import applications.collages.collage;

public class CA {

  public static Integer anyState = new Integer(0);
  public static Integer nonemptyState = new Integer(0);
  
  private static int[][] initial = {{0,0,0},{0,1,0},{0,0,0}};
  
  private int arity;
  private Vector<rule> r = new Vector<rule>();
  
  private int bottom;
  private int left;
  private int[][] cell;
  private NBH neighbourhood = new NBH();
  
  public CA(int arity) {
    this.arity = arity;
    reset();
  }
  
  public int arity() {
    return arity;
  }
  
  public void addRule(Integer top1, Integer top2, Integer top3,
      Integer mid1, Integer mid2, Integer mid3,
      Integer bot1, Integer bot2, Integer bot3,
      Integer result) {
    r.add(new rule(top1,top2,top3,mid1,mid2,mid3,bot1,bot2,bot3,result));
  }
  
  public void reset() {
    cell = initial;
    bottom = left = -1;
  }
  
  private int contents(int row, int col) {
    row -= bottom;
    col -= left;
    if (row < 0 || row >= cell.length || col < 0 || col >= cell[0].length) return 0;
    return cell[row][col];
  }
  
  public void step() {
    int[][] newcell = cell.clone();
    for (int i = 0; i < cell.length; i++) {
      newcell[i] = cell[i].clone();
      for (int j = 0; j < cell[i].length; j++) {
        neighbourhood.setTo(bottom + i, left + j);
        Integer I = match();
        if (I != null) {
          newcell[i][j] = I.intValue();
        }
      }
    }
    cell = newcell;
    complete();
  }
  
  private Integer match() {
    for (int i=0; i < r.size(); i++) {
      Integer I = r.get(i).matches();
      if (I != null) return I;
    }
    return null;
  }
  
  private void complete() {
    int bAdjust = 0;
    int tAdjust = 0;
    int lAdjust = 0;
    int rAdjust = 0;
    for (int j = 0; j<cell[0].length; j++) {
      if (cell[0][j] != 0) bAdjust = -1;
      if (cell[cell.length-1][j] != 0) tAdjust = 1;
    }
    for (int i = 0; i<cell.length; i++) {
      if (cell[i][0] != 0) lAdjust = -1;
      if (cell[i][cell[0].length-1] != 0) rAdjust = 1;
    }
    if (bAdjust == 0 && tAdjust == 0 && lAdjust == 0 && rAdjust == 0) return;
    int[][] newcell = new int[cell.length-bAdjust+tAdjust][cell[0].length-lAdjust+rAdjust];
    for (int j = 0; j<newcell[0].length; j++) newcell[0][j] = newcell[newcell.length-1][j] = 0;
    for (int i = 0; i<newcell.length; i++) newcell[i][0] = newcell[i][newcell[0].length-1] = 0;
    for (int i = 0; i<cell.length; i++) System.arraycopy(cell[i], 0, newcell[i-bAdjust], -lAdjust, cell[0].length);
    cell = newcell;
    bottom += bAdjust;
    left += lAdjust;
  }
  
  public collage apply(Object[] arg) {
    collage result = new collage();
    Rectangle2D.Double[] b = new Rectangle2D.Double[arg.length];
    for (int i = 0; i < b.length; i++) b[i] = ((collage)arg[i]).bounds();
    for (int i = bottom; i < bottom + cell.length; i++) {
      for (int j = left; j < left + cell[0].length; j++) {
        int k = contents(i,j)-1;
        if (k < 0 || k >= arg.length || b[k].width == 0 || b[k].height == 0) continue;
        AffineTransform t = AffineTransform.getScaleInstance(1/b[k].width,1/b[k].height);
        t.concatenate(AffineTransform.getTranslateInstance(-b[k].x, -b[k].y));
        t.preConcatenate(AffineTransform.getTranslateInstance(j,i));
        collage c = (collage)((collage)arg[k]).clone();
        c.transform(t);
        result.unionWith(c);
      }
    }
    return result;
  }
  
  private class NBH {
    private int[][] contents = {{0,0,0},{0,0,0},{0,0,0}};
    public NBH() {}
    public void setTo(int row, int col) {
      row--;
      col--;
      for (int i=0; i<=2; i++)
        for (int j=0; j<=2; j++)
          contents[i][j] = contents(row + i, col + j);
    }
    public int get(int i, int j) {
      return contents[i][j];
    }
  }
  
  private class rule {
    private Integer[][] lhs = new Integer[3][3];
    private Integer rhs;
    public rule(Integer top1, Integer top2, Integer top3, Integer mid1, Integer mid2, Integer mid3, Integer bot1, Integer bot2, Integer bot3, Integer rhs) {
      lhs[0][0] = bot1;
      lhs[0][1] = bot2;
      lhs[0][2] = bot3;
      lhs[1][0] = mid1;
      lhs[1][1] = mid2;
      lhs[1][2] = mid3;
      lhs[2][0] = top1;
      lhs[2][1] = top2;
      lhs[2][2] = top3;
      this.rhs = rhs;
    }
    public Integer matches() {
      boolean dead = true;
      for (int i=0; i < 3; i++) {
        for (int j=0; j < 3; j++) {
          if (neighbourhood.get(i,j) > 0) dead = false;
          if (lhs[i][j] == anyState) continue;
          if (lhs[i][j] == nonemptyState) {
            if (neighbourhood.get(i,j) == 0) return null;
          }
          else if (lhs[i][j].intValue() != neighbourhood.get(i,j)) return null;
        }
      }
      if (dead) return null; else return rhs;
    }
  }

}
